from unittest.mock import patch
import os

from .base_test_class import CNCBaseTestCase
from cnc.models import Application, ProvisionStageManager

from cnc.logger import get_logger

log = get_logger(__name__)


class ProvisionStageSmokeTest(CNCBaseTestCase):
    fixture_name = "backend-1-service"

    def test_tf_is_valid(self):
        app = Application.from_environments_yml(self.env_data_filepath)
        self.assertEqual(len(app.collections), 1)

        collection = app.collections[0]
        collection._infra_outputs_cache = {}
        self.assertEqual(collection.name, "preview")

        self.manager = ProvisionStageManager(collection)
        with patch.object(
            collection.__class__,
            "get_secret_value",
            return_value="secretvalue",
        ):
            self.assertTrue(self.manager.make_ready_for_use())

        self.assertTrue(self.manager.validate())

    def tearDown(self):
        if hasattr(self, "manager"):
            self.manager.cleanup()


class ProvisionStageCleanupSettingsTest(CNCBaseTestCase):
    fixture_name = "backend-1-service"

    def test_provision_cleanup(self):
        app = Application.from_environments_yml(self.env_data_filepath)
        self.assertEqual(len(app.collections), 1)

        collection = app.collections[0]
        collection._infra_outputs_cache = {}
        self.assertEqual(collection.name, "preview")

        manager = ProvisionStageManager(collection)
        self.addCleanup(manager.cleanup)

        manager.make_ready_for_use()
        self.assertEqual(
            sorted(os.listdir(manager.config_files_path)),
            ["_cnc_output", "base.tf.j2", "main.tf.j2", "partials"],
        )

        sample_path = f"{manager.config_files_path}/newfile.txt"
        with open(sample_path, "w") as file:
            file.write("Hello, World!")
        manager.make_ready_for_use()
        self.assertEqual(
            sorted(os.listdir(manager.config_files_path)),
            ["_cnc_output", "base.tf.j2", "main.tf.j2", "partials"],
        )

        with open(sample_path, "w") as file:
            file.write("Hello, World!")
        manager.make_ready_for_use(should_cleanup=False)
        self.assertEqual(
            sorted(os.listdir(manager.config_files_path)),
            ["_cnc_output", "base.tf.j2", "main.tf.j2", "newfile.txt", "partials"],
        )

        with open(f"{manager.config_files_path}/_cnc_output/main.tf", "w") as file:
            file.write("Hello, World!")
        manager.make_ready_for_use(should_cleanup=False, should_regenerate_config=False)
        self.assertEqual(
            open(f"{manager.config_files_path}/_cnc_output/main.tf").read(),
            "Hello, World!",
        )


class AWSProvisionStageSmokeTest(ProvisionStageSmokeTest):
    env_data_filepath = "environments_aws_ecs.yml"


class ProvisionStageTestBase(CNCBaseTestCase):
    fixture_name = "backend-1-service-1-db"

    def setUp(self):
        super().setUp()
        app = Application.from_environments_yml(self.env_data_filepath)
        collection = app.collections[0]
        collection._infra_outputs_cache = {}
        self.manager = ProvisionStageManager(collection)
        with patch.object(
            collection.__class__,
            "get_secret_value",
            return_value="secretvalue",
        ):
            self.assertTrue(self.manager.make_ready_for_use())
        self.assertTrue(self.manager.validate())

        parsed = self.manager.parse_config_file()
        self.resources = parsed.get("resource", {})

    def tearDown(self):
        super().tearDown()
        if hasattr(self, "manager"):
            self.manager.cleanup()


class GCPProvisionStageOneResourceTest(ProvisionStageTestBase):
    def test_database_count(self):
        self.assertEqual(
            len(self.resources.get("google_compute_backend_service", [])), 2
        )
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 2)
        self.assertEqual(
            len(self.resources.get("google_artifact_registry_repository", [])), 1
        )
        self.assertEqual(
            len(self.resources.get("google_compute_managed_ssl_certificate", [])), 1
        )
        self.assertEqual(len(self.resources.get("google_compute_network", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_database", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 1)


class GCPProvisionStageTwoResourcesTest(ProvisionStageTestBase):
    fixture_name = "backend-1-service-2-db"

    def test_database_count(self):
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 2)
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 2)
        self.assertEqual(len(self.resources.get("google_sql_database", [])), 2)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 2)


class GCPProvisionStageTwoServicesTest(ProvisionStageTestBase):
    fixture_name = "backend-2-service-1-db"

    def test_database_count(self):
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 3)
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_database", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 1)


class GCPProvisionStageTwoServicesandDBsTest(ProvisionStageTestBase):
    fixture_name = "backend-2-service-2-db"

    def test_database_count(self):
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 3)
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 2)
        self.assertEqual(len(self.resources.get("google_sql_database", [])), 2)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 2)


class GCPProvisionStageTwoServicesandDBsTwoEnvsTest(ProvisionStageTestBase):
    fixture_name = "backend-1-service-2-db-2-envs"

    def test_database_count(self):
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 3)
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 4)

        log.debug(self.resources.get("google_sql_database"))

        self.assertEqual(len(self.resources.get("google_sql_database", [])), 4)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 4)


class GCPRunLiteProvisionStageOneResourceTest(ProvisionStageTestBase):
    env_data_filepath = "environments_gcp_run_lite.yml"

    def test_database_count(self):
        self.assertIsNone(self.resources.get("google_compute_backend_service"))
        self.assertEqual(len(self.resources.get("google_cloud_run_service", [])), 1)
        self.assertEqual(
            len(self.resources.get("google_artifact_registry_repository", [])), 1
        )
        self.assertIsNone(self.resources.get("google_compute_managed_ssl_certificate"))
        self.assertIsNone(self.resources.get("google_compute_network"))
        self.assertEqual(len(self.resources.get("google_sql_database_instance", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_database", [])), 1)
        self.assertEqual(len(self.resources.get("google_sql_user", [])), 1)


class AWSProvisionStageTestBase(ProvisionStageTestBase):
    env_data_filepath = "environments_aws_ecs.yml"


class AWSProvisionStageOneResourceTest(AWSProvisionStageTestBase):
    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 1)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 1)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 1)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 1)
        self.assertEqual(len(self.resources["aws_vpc"]), 1)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)


class AWSProvisionStageTwoResourcesTest(AWSProvisionStageTestBase):
    fixture_name = "backend-1-service-2-db"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 2)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 2)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 1)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 1)
        self.assertEqual(len(self.resources["aws_vpc"]), 1)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)


class AWSProvisionStageTwoServicesTest(AWSProvisionStageTestBase):
    fixture_name = "backend-2-service-1-db"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 1)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 1)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 2)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 2)
        self.assertEqual(len(self.resources["aws_vpc"]), 1)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)


class AWSProvisionStageTwoServicesandDBsTest(AWSProvisionStageTestBase):
    fixture_name = "backend-2-service-2-db"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 2)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 2)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 2)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 2)
        self.assertEqual(len(self.resources["aws_vpc"]), 1)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)


class AWSProvisionStageTwoServicesandDBsTwoEnvsTest(AWSProvisionStageTestBase):
    fixture_name = "backend-1-service-2-db-2-envs"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 4)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 4)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 2)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 2)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 2)
        self.assertEqual(len(self.resources["aws_vpc"]), 1)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 3)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)


class AWSProvisionStageExistingVpcNoSubnetInfo(AWSProvisionStageTestBase):
    fixture_name = "backend-1-service-1-db"
    env_data_filepath = "environments_aws_ecs_existing_vpc.yml"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 1)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 1)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 1)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 1)
        self.assertEqual(self.resources.get("aws_vpc"), None)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)
        self.assertEqual(len(self.resources["aws_subnet"]), 2)


class AWSProvisionStageExistingVpcCidrsProvided(AWSProvisionStageTestBase):
    fixture_name = "backend-1-service-1-db"
    env_data_filepath = "environments_aws_ecs_existing_vpc_cidrs.yml"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 1)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 1)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 1)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 1)
        self.assertEqual(self.resources.get("aws_vpc"), None)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)
        self.assertEqual(len(self.resources["aws_subnet"]), 2)


class AWSProvisionStageExistingVpcSubnetsProvided(AWSProvisionStageTestBase):
    fixture_name = "backend-1-service-1-db"
    env_data_filepath = "environments_aws_ecs_existing_vpc_subnets.yml"

    def test_tf_is_valid(self):
        self.assertEqual(len(self.resources["aws_db_instance"]), 1)
        self.assertEqual(len(self.resources["aws_db_proxy"]), 1)
        self.assertEqual(len(self.resources["aws_ecs_service"]), 1)
        self.assertEqual(len(self.resources["aws_cloudfront_distribution"]), 1)
        self.assertEqual(len(self.resources["aws_ecr_repository"]), 1)
        self.assertEqual(self.resources.get("aws_vpc"), None)
        self.assertEqual(len(self.resources["aws_acm_certificate"]), 2)
        self.assertEqual(len(self.resources["aws_route53_zone"]), 1)
        self.assertEqual(len(self.resources["aws_subnet"]), 2)
        self.assertEqual(self.resources.get("aws_route_table"), None)



import os
import sys
from pathlib import Path
from dataclasses import dataclass
from typing import Any
from typing_extensions import Annotated

import typer
from rich import print

from .commands import provision, build, deploy, info, shell, toolbox, update
from .commands.template_editor import inspector
from .commands.telemetry import send_event
from .models import Application
from .check_dependencies import check_deps
from cnc.logger import get_logger

log = get_logger(__name__)

app = typer.Typer(pretty_exceptions_show_locals=False)
app.add_typer(provision.app, name="provision")
app.add_typer(build.app, name="build")
app.add_typer(deploy.app, name="deploy")
app.add_typer(inspector.app, name="inspector")
app.add_typer(info.app, name="info")
app.add_typer(shell.app, name="shell")
app.add_typer(toolbox.app, name="toolbox")
app.add_typer(update.app, name="update")


@dataclass
class Common:
    application: Any = None
    collection: Any = None
    environments_file_path = None


@app.callback()
def add_common(
    ctx: typer.Context,
    config_file_path: Annotated[
        Path,
        typer.Option(
            "-f",
            "--config-file-path",
            envvar="CNC_CONFIG_PATH",
            help="CNC config file path",
        ),
    ] = Path("cnc.yml"),
    environments_file_path: Annotated[
        Path,
        typer.Option(
            "-e",
            "--environments-file-path",
            envvar="CNC_ENVIRONMENTS_PATH",
            help="Environments data file path",
        ),
    ] = Path("environments.yml"),
):
    """Common Entry Point for whole app"""
    if "--help" in sys.argv:
        return True

    os.environ["CNC_CONFIG_PATH"] = str(config_file_path)
    os.environ["CNC_ENVIRONMENTS_PATH"] = str(environments_file_path)

    if not config_file_path.is_file():
        log.error(f"Config file not found: {config_file_path}")
        raise typer.Exit(code=1)

    if not environments_file_path.is_file():
        log.error(f"Environments data file not found: {environments_file_path}")
        raise typer.Exit(code=1)

    application = Application.from_environments_yml(
        environments_file_path,
        config_file_path,
    )
    if not application:
        log.error(f"No application for: {environments_file_path}")
        raise typer.Exit(code=1)

    check_deps(application)

    ctx.obj = Common(application)
    ctx.obj.environments_file_path = environments_file_path


@app.command()
def hello():
    send_event("hello")
    print("[bold red]Welcome[/bold red] to [bold purple]" "CNC[/bold purple]")

    raise typer.Exit()


if __name__ == "__main__":
    app()



from .service_account import AmazonAppServiceAccount
from .custom_headers import AWSCustomHeaders
from .database_resource_settings import AWSDatabaseResourceSettings
from .cache_resource_settings import AWSCacheResourceSettings
from .deploy_resource_limits import AWSDeployResourceLimits



from typing import List
import typer
import yaml
from collections import OrderedDict

from cnc.models import ProvisionStageManager
from .telemetry import send_event

from cnc.logger import get_logger

log = get_logger(__name__)


app = typer.Typer()


def ordered_load(stream, Loader=yaml.Loader, object_pairs_hook=OrderedDict):
    class OrderedLoader(Loader):
        pass

    def construct_ordered_mapping(loader, node):
        loader.flatten_mapping(node)
        return object_pairs_hook(loader.construct_pairs(node))

    OrderedLoader.add_constructor(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, construct_ordered_mapping
    )
    return yaml.load(stream, OrderedLoader)


def ordered_dump(data, stream=None, Dumper=yaml.Dumper, **kwds):
    class OrderedDumper(Dumper):
        pass

    def _dict_representer(dumper, data):
        return dumper.represent_dict(data.items())

    OrderedDumper.add_representer(OrderedDict, _dict_representer)
    return yaml.dump(data, stream, OrderedDumper, **kwds)


@app.callback()
def add_common(
    ctx: typer.Context,
    collection_name: str = typer.Option(
        default=None, envvar="CNC_COLLECTION_NAME", help="Collection name"
    ),
):
    """Common Entry Point for Provision"""
    if collection_name:
        collection = ctx.obj.application.collection_by_name(collection_name)
        if not collection:
            log.error(
                f"No collection found for: {collection_name} | (configured "
                f"collections are: {[c.name for c in ctx.obj.application.collections]})"
            )
            raise typer.Exit(code=1)
    elif len(ctx.obj.application.collections) == 1:
        collection = ctx.obj.application.collections[0]
    else:
        log.error(
            f"No collection name provided and more than one collection found for {ctx.obj.application} | (configured "
            f"collections are: {[c.name for c in ctx.obj.application.collections]})"
        )
        raise typer.Exit(code=1)

    ctx.obj.collection = collection


@app.command()
def plan(
    ctx: typer.Context,
    cleanup: bool = True,
    generate: bool = True,
):
    """Generate an infrastructure plan"""
    send_event("provision.plan")
    tf_config = ProvisionStageManager(ctx.obj.collection)
    is_setup = tf_config.make_ready_for_use(
        should_cleanup=cleanup,
        should_regenerate_config=generate,
    )

    if not is_setup:
        log.info(f"Cannot setup {tf_config}")
        raise typer.Exit(code=1)

    try:
        plan = tf_config._tf_command("plan")
        log.info(plan)
    except Exception as e:
        log.error(f"Cannot do TF plan for {tf_config}: {e}")
    finally:
        if cleanup:
            tf_config.cleanup()

    log.debug(f"All set planning for {tf_config.config_files_path}")
    raise typer.Exit()


@app.command()
def apply(
    ctx: typer.Context,
    cleanup: bool = True,
    generate: bool = True,
    update_environments: bool = False,
):
    """Apply an infrastructure plan"""
    send_event("provision.apply")
    tf_config = ProvisionStageManager(ctx.obj.collection)
    is_setup = tf_config.make_ready_for_use(
        should_cleanup=cleanup,
        should_regenerate_config=generate,
    )

    if not is_setup:
        log.info(f"Cannot setup {tf_config}")
        raise typer.Exit(code=1)

    try:
        _ret = tf_config._tf_command("apply")
        log.info(_ret)

        if update_environments:
            # Read environment.yml file into dict
            with open(ctx.obj.environments_file_path, "r") as file:
                environments = ordered_load(file, yaml.SafeLoader)

            # Add to collection.data.infrastructure_outputs if any collections in environments yml dict have the "name" of ctx.obj.collection.name
            for collection in environments.get("collections", []):
                if collection.get("name") == ctx.obj.collection.name:
                    if "data" not in collection:
                        collection["data"] = {}
                    collection["data"][
                        "infrastructure_outputs"
                    ] = ctx.obj.collection.infra_outputs

            # Write back to environments.yml
            with open(ctx.obj.environments_file_path, "w") as file:
                ordered_dump(environments, file, yaml.SafeDumper)

    except Exception as e:
        log.error(f"Cannot do TF apply for {tf_config}: {e}")
    finally:
        if cleanup:
            tf_config.cleanup()

    log.debug(f"All set applying for {tf_config.config_files_path}")
    raise typer.Exit()


@app.command()
def debug(
    ctx: typer.Context,
    cleanup: bool = False,
    generate: bool = True,
):
    """Debug an infrastructure plan"""
    send_event("provision.debug")
    tf_config = ProvisionStageManager(ctx.obj.collection)
    is_setup = tf_config.make_ready_for_use(
        should_cleanup=True,
        should_regenerate_config=generate,
    )

    if not is_setup:
        log.info(f"Cannot setup {tf_config}")
        raise typer.Exit(code=1)

    try:
        tf_config.debug_template_output_directory()
        tf_config.debug_template_output("main.tf")
    except Exception as e:
        log.error(f"Cannot do TF debug for {tf_config}: {e}")
    finally:
        if cleanup:
            tf_config.cleanup()

    log.debug(f"All set debugging for {tf_config}")
    raise typer.Exit()


@app.command()
def cmd(ctx: typer.Context, tf_cmd: List[str]):
    """Run an infrastructure command in the CLI"""
    send_event("provision.cmd")
    tf_config = ProvisionStageManager(ctx.obj.collection)
    is_setup = tf_config.make_ready_for_use()

    if not is_setup:
        log.info(f"Cannot setup {tf_config}")
        raise typer.Exit(code=1)

    try:
        _ret = tf_config._tf_command(tf_cmd[0], args=tf_cmd[1:])
        log.info(_ret)
    except Exception as e:
        log.error(f"Cannot do TF ({tf_cmd}) for {tf_config}: {e}")
    finally:
        tf_config.cleanup()

    log.debug(f"All set running cmd {tf_cmd} for {tf_config.config_files_path}")
    raise typer.Exit()



from typing import Optional, List

from .base_model import BaseModel, IgnoredType


# TODO: Should we use a discriminator here so we
# can have better validations etc?
class ResourceUseExistingSettings(BaseModel):
    name: str
    instance_name: str
    secret_id: Optional[str] = None
    manage_databases: Optional[bool] = True
    username: Optional[str] = None
    db_name: Optional[str] = None
    cluster_mode: Optional[bool] = False
    public_subnet_cidrs: Optional[List[str]] = None
    private_subnet_cidrs: Optional[List[str]] = None
    public_subnet_ids: Optional[List[str]] = None
    private_subnet_ids: Optional[List[str]] = None

    # ------------------------------
    # Parent relationships
    # ------------------------------

    settings: Optional[IgnoredType] = IgnoredType()



import re
from typing import (
    Optional,
    Literal,
    ClassVar,
    List,
)
from pydantic import model_validator
from cnc.models import BaseModel
from cnc.models.custom_header import CustomHeaders, CustomHeader

from cnc.logger import get_logger

log = get_logger(__name__)


class HeaderTypeConfig(BaseModel):
    headers: List[CustomHeader]

    @model_validator(mode="before")
    def filter_headers(cls, data):
        _headers = []
        for header in data.get("headers", []):
            if header["name"] in cls.HEADER_NAMES:
                _headers.append(header)
                cls.update_data_for_header(data, header)

        data["headers"] = _headers

        return data

    @classmethod
    def update_data_for_header(cls, data, header):
        return data


class CorsConfig(HeaderTypeConfig):
    allow_credentials: Optional[bool] = False
    allow_headers: Optional[List[str]] = []
    allow_methods: Optional[List[str]] = []
    allow_origins: Optional[List[str]] = []
    expose_headers: Optional[List[str]] = []
    max_age: Optional[int] = None

    # ------------------------------
    # Properties
    # ------------------------------
    @classmethod
    def update_data_for_header(cls, data, header):
        header_name = header["name"]
        if header_name == "access-control-allow-credentials":
            data["allow_credentials"] = True
        elif header_name == "access-control-allow-headers":
            data["allow_headers"] = re.split(
                AWSCustomHeaders.HEADER_LISTVALUES_REGEX,
                header["value"],
            )
        elif header_name == "access-control-allow-methods":
            data["allow_methods"] = re.split(
                AWSCustomHeaders.HEADER_LISTVALUES_REGEX,
                header["value"],
            )
        elif header_name == "access-control-allow-origin":
            if data["allow_origins"] is None:
                data["allow_origins"] = []
            data["allow_origins"] += re.split(
                AWSCustomHeaders.HEADER_LISTVALUES_REGEX,
                header["value"],
            )
        elif header_name == "access-control-expose-headers":
            data["expose_headers"] = re.split(
                AWSCustomHeaders.HEADER_LISTVALUES_REGEX,
                header["value"],
            )
        elif header_name == "access-control-max-age":
            data["max_age"] = int(header["value"])

        return data

    HEADER_NAMES: ClassVar[List[str]] = [
        "access-control-allow-credentials",
        "access-control-allow-headers",
        "access-control-allow-methods",
        "access-control-allow-origin",
        "access-control-expose-headers",
        "access-control-max-age",
    ]


class StrictTransportConfig(BaseModel):
    max_age: Optional[int] = None
    include_subdomains: Optional[bool] = None
    preload: Optional[bool] = None


class XssConfig(BaseModel):
    mode_block: Optional[bool] = None
    enabled: Optional[bool] = None
    report_uri: Optional[str] = None


class SecurityConfig(HeaderTypeConfig):
    content_security_policy: Optional[str] = None
    frame_option: Optional[str] = None
    referrer_policy: Optional[str] = None
    content_type_options: Optional[bool] = None
    strict_transport_security: Optional[StrictTransportConfig] = None
    xss_protection: Optional[XssConfig] = None

    # ------------------------------
    # Properties
    # ------------------------------
    @classmethod
    def update_data_for_header(cls, data, header):
        header_name = header["name"]
        if header_name == "content-security-policy":
            data["content_security_policy"] = header["value"]
        elif header_name == "x-content-type-options":
            data["content_type_options"] = True
        elif header_name == "x-frame-options":
            data["frame_option"] = header["value"]
        elif header_name == "referrer-policy":
            data["referrer_policy"] = header["value"]
        elif header_name == "strict-transport-security":
            data["strict_transport_directives"] = re.split(
                AWSCustomHeaders.HEADER_DIRECTIVES_REGEX,
                header["value"],
            )
            max_age = None
            include_subdomains = None
            preload = None
            for directive in data["strict_transport_directives"]:
                if directive == "includeSubDomains":
                    include_subdomains = True
                elif directive == "preload":
                    preload = True
                elif "max-age" in directive:
                    max_age = int(directive.split("=")[1])

            data["strict_transport_security"] = {
                "max_age": max_age,
                "include_subdomains": include_subdomains,
                "preload": preload,
            }
        elif header_name == "x-xss-protection":
            xss_directives = re.split(
                AWSCustomHeaders.HEADER_DIRECTIVES_REGEX,
                header["value"],
            )

            mode_block = None
            enabled = False
            report_uri = None
            for directive in xss_directives:
                if directive == "0":
                    enabled = False
                elif directive == "1":
                    enabled = True
                elif directive == "mode=block":
                    mode_block = True
                elif "report=" in directive:
                    report_uri = directive.split("=")[1]

            data["xss_protection"] = {
                "mode_block": mode_block,
                "enabled": enabled,
                "report_uri": report_uri,
            }

        return data

    # ------------------------------
    # Class Vars
    # ------------------------------
    HEADER_NAMES: ClassVar[List[str]] = [
        "content-security-policy",
        "x-content-type-options",
        "x-frame-options",
        "referrer-policy",
        "strict-transport-security",
        "x-xss-protection",
    ]


class AWSCustomHeaders(CustomHeaders):
    provider: Literal["aws"]

    cors_config: CorsConfig
    security_config: SecurityConfig
    custom_headers: List[CustomHeader]

    # ------------------------------
    # Validators
    # ------------------------------
    @model_validator(mode="before")
    def parse_headers_config(cls, data):
        data["cors_config"] = {"headers": data["headers"]}
        data["security_config"] = {"headers": data["headers"]}

        _custom_headers = []
        exclude_from_custom_headers = (
            SecurityConfig.HEADER_NAMES
            + CorsConfig.HEADER_NAMES
            + [cls.SERVER_TIMING_HEADER]
        )
        for header in data["headers"]:
            if header["name"] not in exclude_from_custom_headers:
                _custom_headers.append(header)

        data["custom_headers"] = _custom_headers

        return data

    # ------------------------------
    # Class vars
    # ------------------------------
    SERVER_TIMING_HEADER: ClassVar[str] = "server-timing"
    HEADER_LISTVALUES_REGEX: ClassVar[str] = ",\\s*"
    HEADER_DIRECTIVES_REGEX: ClassVar[str] = ";\\s*"



from unittest.mock import patch

from .base_test_class import CNCBaseTestCase
from cnc.models import Application, BuildStageManager
from cnc.models.providers.google.environment_collection import GCPEnvironmentCollection


from cnc.logger import get_logger

log = get_logger(__name__)


class BuildTemplateConfigTest(CNCBaseTestCase):
    fixture_name = "backend-1-service"

    def test_template_config(self):
        app = Application.from_environments_yml("environments.yml")
        self.assertEqual(app.template_config.enabled, False)

        app = Application.from_environments_yml("environments-custom-provision.yml")
        self.assertEqual(app.template_config.enabled, True)
        self.assertEqual(app.template_config.template_directory, "custom")
        self.assertEqual(app.template_config.provision_filename, "main.tf.j2")
        self.assertEqual(app.template_config.deploy_filename, "main.sh.j2")
        self.assertEqual(app.template_config.build_filename, "main.sh.j2")


class BuildTagsPerServiceTestCase(CNCBaseTestCase):
    fixture_name = "backend-2-service-1-db"

    def test_template_config_tags(self):
        app = Application.from_environments_yml("environments.yml")
        environment = app.collections[0].environments[0]
        svc1 = environment.web_services[0]
        svc2 = environment.web_services[1]

        builder = BuildStageManager(environment, {svc1.name: "tag1", svc2.name: "tag2"})
        builder.cleanup()
        builder.setup()

        with patch.object(
            GCPEnvironmentCollection, "get_secret_value", return_value="secretvalue"
        ):
            builder.render_build()

        with open(f"{builder.rendered_files_path}/build-app-functions.sh", "r") as file:
            app_build_functions = file.read()

        with open(f"{builder.rendered_files_path}/build-api-functions.sh", "r") as file:
            api_build_functions = file.read()

        self.assertIn(
            f"us-east1-docker.pkg.dev/foo-bar-123/{environment.collection.instance_name}/my-backend-t-app:tag1",
            app_build_functions,
        )
        self.assertIn(
            f"us-east1-docker.pkg.dev/foo-bar-123/{environment.collection.instance_name}/my-backend-t-api:tag2",
            api_build_functions,
        )

        builder.cleanup()


class GCPBuildStageTestBase(CNCBaseTestCase):
    fixture_name = "backend-1-service-1-db"
    environment_name = "main"

    def setUp(self):
        super().setUp()
        app = Application.from_environments_yml("environments.yml")
        collection = app.collections[0]

        self.environment = collection.environment_by_name(self.environment_name)
        self.assertEqual(self.environment_name, self.environment.name)

        self.manager = BuildStageManager(self.environment)

        self.manager.cleanup()
        self.manager.setup()
        with patch.object(
            GCPEnvironmentCollection, "get_secret_value", return_value="secretvalue"
        ):
            self.manager.render_build()

    def tearDown(self):
        super().tearDown()
        if hasattr(self, "manager"):
            self.manager.cleanup()

    def parse(self, script_name="build.sh"):
        with open(f"{self.manager.rendered_files_path}/{script_name}", "r") as file:
            return file.read()


class GCPBuildStageSmokeTest(GCPBuildStageTestBase):
    def test_build_functions(self):
        build = self.parse("build-app.sh")
        self.assertNotIn("run_image", build)
        self.assertNotIn("verify_app_image_exists", build)
        self.assertIn("build_app_image", build)

        build_functions = self.parse("build-app-functions.sh")
        self.assertNotIn("run_image", build)
        self.assertIn("verify_app_image_exists", build_functions)
        self.assertIn("build_app_image ()", build_functions)
        self.assertIn("build_app_image ()", build_functions)


class AWSBuildStageTestBase(CNCBaseTestCase):
    fixture_name = "backend-1-service-1-db"
    environment_name = "main"
    env_data_filepath = "environments_aws_ecs.yml"

    def setUp(self):
        super().setUp()
        app = Application.from_environments_yml(self.env_data_filepath)
        collection = app.collections[0]

        self.environment = collection.environment_by_name(self.environment_name)
        self.assertEqual(self.environment_name, self.environment.name)

        self.manager = BuildStageManager(self.environment)

        self.manager.cleanup()
        self.manager.setup()
        self.manager.render_build()

    def tearDown(self):
        super().tearDown()
        if hasattr(self, "manager"):
            self.manager.cleanup()

    def parse(self, script_name="build.sh"):
        with open(f"{self.manager.rendered_files_path}/{script_name}", "r") as file:
            return file.read()


class AWSBuildStageSmokeTest(AWSBuildStageTestBase):
    def test_build_functions(self):
        self.assertTrue(self.environment.application.provider_is_aws)

        build = self.parse("build-app.sh")
        self.assertNotIn("run_image", build)
        self.assertNotIn("verify_app_image_exists", build)
        self.assertIn("build_app_image", build)

        build_functions = self.parse("build-app-functions.sh")
        self.assertNotIn("run_image", build_functions)
        self.assertNotIn("verify_app_image_exists", build_functions)
        self.assertIn("build_app_image ()", build_functions)


class AWSExistingImageBuildStageTestCase(AWSBuildStageTestBase):
    fixture_name = "backend-1-service-existing"

    def test_build_functions(self):
        self.assertTrue(self.environment.application.provider_is_aws)

        build = self.parse("build-app.sh")
        self.assertNotIn("run_image", build)
        self.assertIn("verify_app_image_exists", build)
        self.assertNotIn("build_app_image", build)

        build_functions = self.parse("build-app-functions.sh")
        self.assertNotIn("run_image", build_functions)
        self.assertIn("verify_app_image_exists", build_functions)


class GCPExistingImageBuildStageTestCase(GCPBuildStageTestBase):
    fixture_name = "backend-1-service-existing"

    def parse(self, script_name="build.sh"):
        with open(f"{self.manager.rendered_files_path}/{script_name}", "r") as file:
            return file.read()

    def test_build_functions(self):
        self.assertTrue(self.environment.application.provider_is_gcp)

        build = self.parse("build-app.sh")
        self.assertNotIn("run_image", build)
        self.assertIn("verify_app_image_exists", build)
        self.assertNotIn("build_app_image", build)

        build_functions = self.parse("build-app-functions.sh")
        self.assertNotIn("run_image", build_functions)
        self.assertIn("verify_app_image_exists", build_functions)



from unittest.mock import patch

from .base_test_class import CNCBaseTestCase
from cnc.models import Application
from cnc.models.providers.google.environment_collection import GCPEnvironmentCollection

from cnc.logger import get_logger

log = get_logger(__name__)


class EnvironmentVariablesTestCase(CNCBaseTestCase):
    fixture_name = "backend-1-service-1-db-variables-aliases"

    def setUp(self):
        super().setUp()
        app = Application.from_environments_yml("environments.yml")
        self.collection = app.collections[0]
        self.environment = self.collection.environments[0]
        self.assertEqual(self.environment.name, "main")

    def test_variables(self):
        self.assertEqual(len(self.environment.environment_variables), 6)

        _std = self.environment.environment_variables[0]
        self.assertEqual(_std.name, "foo-standard")
        self.assertEqual(_std.value, "bar")
        self.assertEqual(_std.secret_id, None)
        self.assertEqual(_std.output_name, None)
        self.assertEqual(_std.alias, None)

        _secret = self.environment.environment_variables[1]
        with patch.object(
            GCPEnvironmentCollection, "get_secret_value", return_value="secretvalue"
        ) as secret_mock:
            self.assertEqual(_secret.value, "secretvalue")
            secret_mock.assert_called_once

        self.assertEqual(_secret.secret_id, "bar123")
        self.assertEqual(_secret.output_name, None)
        self.assertEqual(_secret.alias, None)

        _std_alias = self.environment.environment_variables[3]
        self.assertEqual(_std_alias.name, "foo-standard-alias")
        self.assertEqual(_std_alias.secret_id, None)
        self.assertEqual(_std_alias.output_name, None)
        self.assertEqual(_std_alias.alias, "foo-standard")
        self.assertEqual(_std.value, "bar")

    def test_items_property(self):
        item_names = [item.name for item in self.environment.environment_items]

        # variable
        self.assertIn("foo-standard", item_names)
        # managed var
        self.assertIn("DB_PASSWORD", item_names)
        # managed secret
        self.assertIn("DB1_ENDPOINT", item_names)

        # check that there are no dupe names
        self.assertEqual(len(item_names), len(set(item_names)))



import os
from inspect import currentframe
from rich import print


def get_logger(name="root"):
    return Logger(name)


class Logger:
    def __init__(self, name):
        self.name = name

    def debug(self, msg):
        debug = bool(os.environ.get("CNC_DEBUG", True))
        if debug:
            print(
                f"[green]DEBUG ({self.name}:{currentframe().f_back.f_lineno})"
                f"[/ green] {msg}"
            )

    def info(self, msg):
        print(
            f"[blue]INFO ({self.name}:{currentframe().f_back.f_lineno})"
            f"[/ blue] {msg}"
        )

    def warning(self, msg):
        print(
            f"[yellow]WARNING ({self.name}:{currentframe().f_back.f_lineno})"
            f"[/ yellow] {msg}"
        )

    def error(self, msg):
        print(
            f"[red]ERROR ({self.name}:{currentframe().f_back.f_lineno})"
            f"[/ red] {msg}"
        )



import re
import json
from typing import Literal

from cnc.models.config.resource import DatabaseResourceSettings
from cnc.logger import get_logger

log = get_logger(__name__)


class AWSDatabaseResourceSettings(DatabaseResourceSettings):
    provider: Literal["aws"]

    # ------------------------------
    # Properties
    # ------------------------------

    @property
    def managed_secret_id(self):
        return f"{self.environment.instance_name}-secrets"

    @property
    def url_secret_id(self):
        return f"{self.managed_secret_id}:{self.service.instance_name}_url::"

    @property
    def password_secret_id(self):
        return f"{self.managed_secret_id}:{self.service.instance_name}_password::"

    @property
    def database_credentials_secret_id(self):
        if self.use_existing:
            return self.use_existing.secret_id

        return f"{self.service.instance_name}-db-creds"

    @property
    def host_output_tf_value_string(self):
        resource_type_fragment = "${"

        if self.use_db_proxy:
            resource_type_fragment += "aws_db_proxy"
        elif self.use_existing:
            if self.cluster_mode:
                resource_type_fragment += "data.aws_rds_cluster"
            else:
                resource_type_fragment += "data.aws_db_instance"
        else:
            resource_type_fragment += "aws_db_instance"

        return "".join(
            [f"{resource_type_fragment}.", self.service.instance_name, ".endpoint}"]
        )

    @property
    def password_from_db_secret(self):
        try:
            db_secret_value = self.collection.get_secret_value(
                self.database_credentials_secret_id
            )
            return json.loads(db_secret_value).get("password")
        except Exception as e:
            log.debug(f"No existing db secret found for {self.service}: {e}")
            return None

    @property
    def engine_family(self):
        engine_to_family = {
            "mysql": "MYSQL",
            "sqlserver": "SQLSERVER",
            "postgres": "POSTGRESQL",
        }

        return engine_to_family[self.engine.lower()]

    @property
    def identifier(self):
        if self.use_existing:
            return self.service.instance_name

        identifier = re.sub("-+", "-", self.service.instance_name)
        identifier = re.sub("cursor", "cvrsor", identifier)

        return identifier

    @property
    def license_model(self):
        if self.engine.lower() == "sqlserver":
            return "license-included"

    @property
    def instance_class(self):
        if self.engine.lower() == "sqlserver":
            return "db.t3.small"

        return "db.t3.micro"

    @property
    def db_name(self):
        if self.engine.lower() != "sqlserver":
            return super().db_name

    @property
    def cluster_mode(self):
        return bool(self.use_existing and self.use_existing.cluster_mode)

    @property
    def toolbox_ssh_port_mapping(self):
        return (
            "{"
            f'"host":["{self.database_endpoint}"],"portNumber":'
            f'["{self.remote_port}"], "localPortNumber":["{self.local_port}"]'
            "}"
        )



from typing import Literal

from cnc.models.config.resource import CacheResourceSettings
from cnc.logger import get_logger

log = get_logger(__name__)


class AWSCacheResourceSettings(CacheResourceSettings):
    provider: Literal["aws"]

    @property
    def toolbox_ssh_port_mapping(self):
        return (
            "{"
            f'"host":["{self.redis_ip}"],"portNumber":["{self.redis_port}"], '
            f'"localPortNumber":["{self.local_port}"]'
            "}"
        )
